/* tune-s2 -- simple zapping tool for the Linux DVB S2 API
 *
 * UDL (updatelee@gmail.com)
 * Derived from work by:
 * 	Igor M. Liplianin (liplianin@me.by)
 * 	Alex Betis <alex.betis@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <limits.h>
#include <errno.h>
#include <fcntl.h>
#include <time.h>
#include <signal.h>
#include <stdio.h>
#include <stdint.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/poll.h>
#include <sys/param.h>
#include <sys/time.h>
#include <linux/dvb/frontend.h>
#include <linux/dvb/version.h>
#include <termios.h>

#include "diseqc.h"

#define FEC_S2_QPSK_1_2 (fe_code_rate_t)(FEC_AUTO+1)
#define FEC_S2_QPSK_2_3 (fe_code_rate_t)(FEC_S2_QPSK_1_2+1)
#define FEC_S2_QPSK_3_4 (fe_code_rate_t)(FEC_S2_QPSK_2_3+1)
#define FEC_S2_QPSK_5_6 (fe_code_rate_t)(FEC_S2_QPSK_3_4+1)
#define FEC_S2_QPSK_7_8 (fe_code_rate_t)(FEC_S2_QPSK_5_6+1)
#define FEC_S2_QPSK_8_9 (fe_code_rate_t)(FEC_S2_QPSK_7_8+1)
#define FEC_S2_QPSK_3_5 (fe_code_rate_t)(FEC_S2_QPSK_8_9+1)
#define FEC_S2_QPSK_4_5 (fe_code_rate_t)(FEC_S2_QPSK_3_5+1)
#define FEC_S2_QPSK_9_10 (fe_code_rate_t)(FEC_S2_QPSK_4_5+1)
#define FEC_S2_8PSK_1_2 (fe_code_rate_t)(FEC_S2_QPSK_9_10+1)
#define FEC_S2_8PSK_2_3 (fe_code_rate_t)(FEC_S2_8PSK_1_2+1)
#define FEC_S2_8PSK_3_4 (fe_code_rate_t)(FEC_S2_8PSK_2_3+1)
#define FEC_S2_8PSK_5_6 (fe_code_rate_t)(FEC_S2_8PSK_3_4+1)
#define FEC_S2_8PSK_7_8 (fe_code_rate_t)(FEC_S2_8PSK_5_6+1)
#define FEC_S2_8PSK_8_9 (fe_code_rate_t)(FEC_S2_8PSK_7_8+1)
#define FEC_S2_8PSK_3_5 (fe_code_rate_t)(FEC_S2_8PSK_8_9+1)
#define FEC_S2_8PSK_4_5 (fe_code_rate_t)(FEC_S2_8PSK_3_5+1)
#define FEC_S2_8PSK_9_10 (fe_code_rate_t)(FEC_S2_8PSK_4_5+1)

struct lnb_p
{
	int low;
	int high;
	int threshold;
};

struct tune_p
{
	int freq;
	int sr;
	int fec;
	int inversion;
	int voltage;
	int tone;
	int LO;
};

struct options
{
	char *name;
	int value;
};

struct options dvb_fec[] =
{
	{ "1/2", FEC_1_2 },
	{ "2/3", FEC_2_3 },
	{ "3/4", FEC_3_4 },
	{ "4/5", FEC_4_5 },
	{ "5/6", FEC_5_6 },
	{ "6/7", FEC_6_7 },
	{ "7/8", FEC_7_8} ,
	{ "8/9", FEC_8_9 },
	{ "AUTO", FEC_AUTO },
	{ "S2_QPSK_1/2", FEC_S2_QPSK_1_2 },
	{ "S2_QPSK_2/3", FEC_S2_QPSK_2_3 },
	{ "S2_QPSK_3/4", FEC_S2_QPSK_3_4 },
	{ "S2_QPSK_5/6", FEC_S2_QPSK_5_6 },
	{ "S2_QPSK_7/8", FEC_S2_QPSK_7_8 },
	{ "S2_QPSK_8/9", FEC_S2_QPSK_8_9 },
	{ "S2_QPSK_3/5", FEC_S2_QPSK_3_5 },
	{ "S2_QPSK_4/5", FEC_S2_QPSK_4_5 },
	{ "S2_QPSK_9/10", FEC_S2_QPSK_9_10 },
	{ "S2_8PSK_1/2", FEC_S2_8PSK_1_2 },
	{ "S2_8PSK_2/3", FEC_S2_8PSK_2_3 },
	{ "S2_8PSK_3/4", FEC_S2_8PSK_3_4 },
	{ "S2_8PSK_5/6", FEC_S2_8PSK_5_6 },
	{ "S2_8PSK_7/8", FEC_S2_8PSK_7_8 },
	{ "S2_8PSK_8/9", FEC_S2_8PSK_8_9 },
	{ "S2_8PSK_3/5", FEC_S2_8PSK_3_5 },
	{ "S2_8PSK_4/5", FEC_S2_8PSK_4_5 },
	{ "S2_8PSK_9/10", FEC_S2_8PSK_9_10 },
	{ NULL, 0 }
};

struct options dvb_inversion[] =
{
	{ "OFF", INVERSION_OFF },
	{ "ON", INVERSION_ON },
	{ "AUTO", INVERSION_AUTO },
	{ NULL, 0 }
};

struct options dvb_voltage[] =
{
	{ "V", SEC_VOLTAGE_13 },
	{ "H", SEC_VOLTAGE_18 },
	{ "R", SEC_VOLTAGE_13 },
	{ "L", SEC_VOLTAGE_18 },
	{ NULL, 0 }
};

struct options dvb_tone[] =
{
	{ "ON", SEC_TONE_ON },
	{ "OFF", SEC_TONE_OFF },
	{ NULL, 0 }
};	

int name2value(char *name, struct options *table);
char * value2name(int value, struct options *table);
